import json
import os

from jupyter_server.base.handlers import APIHandler
from jupyter_server.utils import url_path_join
import tornado


from .oslnotify_lib import calendar
from .oslnotify_lib import storage

class RouteHandler(APIHandler):
    # The following decorator should be present on all verb methods (head, get, post,
    # patch, put, delete, options) to ensure only authorized user can request the
    # Jupyter server
    @tornado.web.authenticated
    def get(self):
        """
        /opensarlab-frontend/opensarlab-oslnotify?type=calendar,storage
        """
        note_type = self.get_query_argument('type')
        note_type = note_type.split(',')

        profile_name = os.environ.get('OPENSARLAB_PROFILE_NAME', '')
        lab_short_name = os.environ.get('OPENSCIENCELAB_LAB_SHORT_NAME', '')
        portal_domain = os.environ.get('OPENSCIENCELAB_PORTAL_DOMAIN', '')

        events = []

        if 'calendar' in note_type:
            events += calendar.main(profile_name, lab_short_name, portal_domain)

        if 'storage' in note_type:
            events += storage.main()

        options = {
            'closeButton': 'true',
            'newestOnTop': 'true',
            'progressBar': 'true',
            'positionClass': 'toast-bottom-right',
            'preventDuplicates': 'false',
            'onclick': 'undefined',
            'showDuration': '30',
            'hideDuration': '1',
            'timeOut': '0',
            'extendedTimeOut': '0',
            'showEasing': 'swing',
            'hideEasing': 'linear',
            'showMethod': 'fadeIn',
            'hideMethod': 'fadeOut'
        }

        if any([e.get('severity', '-1') for e in events]) == 1:
            options['positionClass'] = 'toast-top-full-width'

        self.finish(json.dumps({"data": events, "options": options}))

def setup_handlers(base_url, url_path=None):
    route_pattern = url_path_join(base_url, "opensarlab-oslnotify")
    return [(route_pattern, RouteHandler)]



import requests

def main(profile_name: str='default', lab_short_name: str='default', portal_domain: str="https://example.com"):

    try:
        active_events = []
        
        # Get calendar notifications
        resp = requests.get(f"{portal_domain}/user/notifications/{lab_short_name}?profile={profile_name}")
        resp = resp.json()

        for event in resp:

            type_event = event.get('type', 'success')
            if type_event == 'error':
                severity = 1
            else:
                severity = 0

            active_events.append(
                {
                    "title": event.get('title', 'Notification'),
                    "message": event.get('message', ''),
                    "type": type_event,
                    "severity": severity
                }
            )

        return active_events

    except Exception as e:
        print(e)
        raise Exception(f"{e}")



import json
import os

from jupyter_server.base.handlers import APIHandler
from jupyter_server.utils import url_path_join
import tornado

class RouteHandler(APIHandler):
    # The following decorator should be present on all verb methods (head, get, post,
    # patch, put, delete, options) to ensure only authorized user can request the
    # Jupyter server
    @tornado.web.authenticated
    def get(self):
        """
        /opensarlab-frontend/opensarlab-profile-label
        """
        profile_name = os.environ.get('OPENSARLAB_PROFILE_NAME', 'No environment variable "OPENSARLAB_PROFILE_NAME" found!')
        self.finish(json.dumps({"data": profile_name}))

def setup_handlers(base_url, url_path=None):
    route_pattern = url_path_join(base_url, "opensarlab-profile-label")
    return [(route_pattern, RouteHandler)]



import shutil
import pathlib

def main(df_path: pathlib.Path=pathlib.Path.home()):

    try:
        active_events = []

        # Get storage usage
        res = shutil.disk_usage(df_path)

        GB = 1.0/1024/1024/1024
        storage_usage = f"{res.used * GB:.2f}"
        storage_total = f"{res.total * GB:.2f}"
        #storage_free = f"{res.free * GB:.2f}"
        storage_percent = f"{res.used / res.total * 100.:.2f}"
        storage_percent_raw = res.used / res.total * 100.

        event_name = 'Current Storage Usage'
        if storage_percent_raw > 99:
            event_type = 'error'
            severity = 1
            message = f"""
                <h1>Current storage usage is at {storage_percent}% ({storage_usage} / {storage_total} GB).</h1>
                <h2>Please delete any unused files. Failure to do so may make your server unresponsive.</h2>
            """

        elif storage_percent_raw > 90:
            event_type = 'warning'
            severity = 0
            message = f"<p>Current storage usage is at {storage_percent}% ({storage_usage} / {storage_total} GB)</p>"

        else:
            event_type = 'success'
            severity = 0
            message = f"<p>Current storage usage is at {storage_percent}% ({storage_usage} / {storage_total} GB)</p>"

        active_events.append(
            {
                "title": event_name,
                "message": message,
                "type": event_type, 
                'severity': severity
            }
        )

        return active_events

    except Exception as e:
        print(e)
        raise Exception(f"{e}")



import json

from jupyter_server.utils import url_path_join

def setup_handlers(web_app, url_path=None):
    base_url = web_app.settings["base_url"]
    base_url = url_path_join(base_url, "opensarlab-frontend")
    handlers = []

    # OSL Notify
    from .handles.oslnotify import setup_handlers as setup_oslnotify
    handlers += setup_oslnotify(base_url)
    
    # Profile Label
    from .handles.profile_label import setup_handlers as setup_profile_label
    handlers += setup_profile_label(base_url)

    host_pattern = ".*$"
    web_app.add_handlers(host_pattern, handlers) 